@@ -35,6 +35,8 @@ else
  'compress' => '/usr/cs/contrib/bin/gzip',
  'bzip' => undef,
  'bzip2' => undef,
+ 'lzip' => undef,
+ 'xz' => undef,
 );
 
 # ---------------------------------------------------------------------------
