@@ -14,6 +14,9 @@ use FileHandle;
 use Test::ConfigureGrepmail;
 use File::Copy;
 use File::Spec;
+use Config;
+
+my $path_to_perl = $Config{perlpath};
 
 BEGIN
 {
@@ -24,6 +27,11 @@ BEGIN
 my $MAILBOX_SIZE = 10_000_000;
 my $TEMP_MAILBOX = 't/temp/bigmailbox.txt';
 
+my $SKIP = 5;
+my $MINIMUM = 10;
+my $CONFIDENCE = 99;
+my $ERROR = 1;
+
 my @IMPLEMENTATIONS_TO_TEST = (
 'Perl',
 'Grep',
@@ -32,29 +40,32 @@ my @IMPLEMENTATIONS_TO_TEST = (
 );
 
 my %TESTS = (
-'SIMPLE' => "grepmail library $TEMP_MAILBOX",
-#'DATE' => "grepmail library -d \"before oct 15 1998\" $TEMP_MAILBOX",
-'COMPRESSED' => "grepmail library $TEMP_MAILBOX.gz",
-#'HEADER' => "grepmail -h library $TEMP_MAILBOX",
-#'BODY' => "grepmail -b library $TEMP_MAILBOX",
-#'BODY & HEADER' => "grepmail -bh library $TEMP_MAILBOX",
-#'PIPE' => "cat $TEMP_MAILBOX | grepmail library",
+'SIMPLE' => "grepmail library \$TEMP_MAILBOX",
+#'DATE' => "grepmail library -d \"before oct 15 1998\" \$TEMP_MAILBOX",
+'COMPRESSED' => "grepmail library \$TEMP_MAILBOX",
+#'HEADER' => "grepmail -h library \$TEMP_MAILBOX",
+#'BODY' => "grepmail -b library \$TEMP_MAILBOX",
+#'BODY & HEADER' => "grepmail -bh library \$TEMP_MAILBOX",
+#'PIPE' => "cat \$TEMP_MAILBOX | grepmail library",
 );
 
 mkdir 't/temp';
 
-CreateInputFiles($TEMP_MAILBOX);
+my $filename = CreateInputFiles();
 
 foreach my $label (keys %TESTS)
 {
   print "\n";
 
-  print "Executing speed test \"$label\":\n$TESTS{$label}\n\n";
+  my $new_filename = $filename;
+  $new_filename .= ".gz" if $label =~ /COMPRESS/;
 
-  my $mailbox = $TEMP_MAILBOX;
-  $mailbox .= '.gz' if $label =~ /COMPRESS/;
+  my $test = $TESTS{$label};
+  $test =~ s/\$TEMP_MAILBOX/$new_filename/g;
 
-  my $data = CollectData($mailbox, $TESTS{$label});
+  print "Executing speed test \"$label\":\n$test\n\n";
+
+  my $data = CollectData($test);
 
   print "=========================================\n";
 
@@ -86,10 +97,12 @@ sub RemoveInputFile
 
 sub CreateInputFiles
 {
-  my $filename = shift;
+  my $filename = $TEMP_MAILBOX;
 
   my @mailboxes;
 
+  my @letters = 'a'..'z';
+
   unless(-e $filename && abs((-s $filename) - $MAILBOX_SIZE) <= $MAILBOX_SIZE*.1)
   {
     print "Making input file ($MAILBOX_SIZE bytes).\n";
@@ -142,7 +155,10 @@ Content-Disposition: attachment; filename="foo.tar.gz"
 
 EOF
 
-      print FILE (('x' x 74 . "\n" ) x (1_000_000 / 74));
+      for (1 .. 1_000_000/75) {
+        print FILE @letters[rand(26)] for 1..74;
+        print FILE "\n";
+      }
 
       print FILE "--873612032-418625252-1050794990=:31078--\n\n";
     }
@@ -150,20 +166,19 @@ EOF
     close FILE;
   }
 
-  unlink "$filename.gz" if -e "$filename.gz";
-
   print "Making compressed input file.\n";
 
+  unlink "$filename.gz";
+
   system "gzip -c --force --best $filename > $filename.gz";
 
-  return ($filename, "$filename.gz");
+  return $filename;
 }
 
 ################################################################################
 
 sub CollectData
 {
-  my $filename = shift;
   my $test = shift;
 
   print "Collecting data...\n\n";
@@ -188,7 +203,7 @@ sub CollectData
     'Cache Use' => [1,0],
   );
 
-  foreach my $old_or_new qw(New Old)
+  foreach my $old_or_new (qw(New Old))
   {
     my $grepmail = 't/temp/grepmail';
     $grepmail .= '.old' if $old_or_new eq 'Old';
@@ -200,18 +215,19 @@ sub CollectData
       my $label = "$old_or_new $impl";
 
       my $new_test = $test;
-      $new_test =~ s/\bgrepmail\b/$^X $grepmail/g;
+      $new_test =~ s/\bgrepmail\b/$path_to_perl $grepmail/g;
 
       print "$impl ($old_or_new)\n";
 
       Test::ConfigureGrepmail::Set_Caching_And_Grep($grepmail,
         @{$settings{$impl}});
 
-      my $t = new Benchmark::Timer(skip => 1, confidence => 97.5, error => 2);
+      my $t = new Benchmark::Timer(skip => $SKIP, minimum => $MINIMUM,
+        confidence => $CONFIDENCE, error => $ERROR);
 
       # Need enough for the statistics to be valid
-      my $count = 0;
-      while ($count - 1 < 10 || $t->need_more_samples($label))
+      my $count = 1;
+      while ($t->need_more_samples($label))
       {
         print ".";
 
@@ -229,11 +245,16 @@ sub CollectData
 
       print $t->report($label);
 
+      my $sum = 0;
+      my %results = $t->data();
+      map { $sum += $_ } @{$results{$label}};
+
       # Fake a benchmark object so we can compare later using Benchmark
-      $data{$label} = new Benchmark;
-      $data{$label}[5] = 1;
-      $data{$label}[1] = $t->result($label);
-      $data{$label}[2] = 0;
+      my $benchmark = new Benchmark;
+      die "Benchmark object doesn't look right" unless @$benchmark == 6;
+      # Assign our total to the CPU entry, since that what the module compares.
+      @$benchmark = ( $sum, 0, 0, $sum, 0, scalar @{$results{$label}} );
+      $data{$label} = $benchmark;
     }
   }
 
